﻿/*
 * Készítette a SharpDevelop.
 * Felhasználó: phil
 * Dátum: 2009.07.03.
 * Idő: 12:14
 * 
 * A sablon megváltoztatásához használja az Eszközök | Beállítások | Kódolás | Szabvány Fejlécek Szerkesztését.
 */
using System;
using System.Collections;
using System.Collections.Generic;

namespace Noise.DataAcquisition
{
	/// <summary>
	/// A class to distribute data between several reader queues (eg, one for writing into file, the other for charting).
	/// It has an input queue and a list of output queues. The input queue stores voltages, whereas the output queues may
	/// have the voltage values converted into other physical quantitites if the appropriate SensorConverter is defined.
	/// </summary>
	public class DataPipe : List<Queue<double[]>>
	{
		protected Sensor[] sensors;					// One per channel.
		protected volatile bool conversionActive = false;
		
		
		public DataPipe(IEnumerable<Queue<double[]>> queues, Sensor[] sensors) : base(queues)
		{
			this.sensors = sensors;
		}
		
		public DataPipe(Sensor[] sensors) : base()
		{
			this.sensors = sensors;
		}
		
		public DataPipe() : this(null) { }
		
		
		public bool ConversionActive {
			get { return conversionActive; }
			set { conversionActive = value; }
		}
		
		public Sensor[] Sensors {
			get { return sensors; }
			set { sensors = value; }
		}
		
		
		public void ClearAll()
		{		
			foreach (Queue<double[]> queue in this) {
				lock ((queue as ICollection).SyncRoot) {
					queue.Clear();
				}
			}
		}
		
		public double[] Convert(double[] voltages)
		{
			if (sensors == null) return voltages;
			if (sensors.Length < voltages.Length) return voltages;
			double[] converted = new double[voltages.Length];
			
			for (int i = 0; i < converted.Length; i++) {
				converted[i] = sensors[i].GetValue(voltages[i]);
			}
			
			return converted;
		}
		
		public void EnqueueAll(double[] item)
		{
			double[] itemToEnqueue = conversionActive ? Convert(item) : item;
			
			foreach (Queue<double[]> queue in this) {
				lock ((queue as ICollection).SyncRoot) {
					queue.Enqueue(itemToEnqueue);
				}
			}
		}
	}
}
